/*
   This program demonstrates the use of the glReadPixels() function.
   See
       http://www.opengl.org/sdk/docs/man/xhtml/glReadPixels.xml
*/
#include <stdio.h>
#include <math.h>

#if defined _WIN32 || defined _WIN64
#  include <windows.h>
#endif

#ifdef __APPLE__
#  include <GLUT/glut.h>
#else
#  include <GL/glut.h>
#endif

int windowHeight;         // this is set by the glut reshape() callback function
int windowWidth;          // this is set by the glut reshape() callback function

#define rowCount 20
#define colCount 20

float boxColor[rowCount][colCount][3];  // array to hold randmly chosen colors
float bigBoxR, bigBoxG, bigBoxB;


/* Prototypes for the GLUT callback functions */
void display( void );
void reshape(int, int);
void mouse(int, int, int, int);

int main(int argc, char** argv)
{
   int i,j;

   /* Standard GLUT initialization */
   glutInit(&argc,argv);
   glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB); /* default, not needed */
   glutInitWindowSize(400, 200);
   glutInitWindowPosition(100, 100);
   glutCreateWindow("glReadPixels()");
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   glutMouseFunc(mouse);

   /* Initialize OpenGL state */
   /* attributes */
   glClearColor(1.0, 1.0, 1.0, 1.0); /* white background */

   // initialize the array of random colors
   for (i = 0; i < rowCount; i++)
   {
      for (j = 0; j < colCount; j++)
      {
         boxColor[i][j][0] = abs(rand()%10000)/10000.0;
         boxColor[i][j][1] = abs(rand()%10000)/10000.0;
         boxColor[i][j][2] = abs(rand()%10000)/10000.0;
      }
   }

   /* Enter event loop */
   glutMainLoop();

   return 0;  /* we never get here */
}//main()



/* The display() callback function */
void display( void )
{
   float boxW, boxH;
   int i, j;

   glClear(GL_COLOR_BUFFER_BIT); /* clear the window */

   // viewport
   glViewport(0, 0, windowWidth, windowHeight);

   /* set up viewing */
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(0.0, windowWidth, 0.0, windowHeight);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   boxW = (float)windowWidth/(2.0 * colCount);
   boxH = (float)windowHeight/(float)rowCount;

   // Draw the grid of randomly colored boxes on
   // the left half of the screen.
   for (i = 0; i < rowCount; i++)
   {
      for (j = 0; j < colCount; j++)
      {  // This should be done with tranformations!
         glColor3fv( boxColor[i][j] );  // use a color vector
         glBegin(GL_QUADS);
           glVertex2f(     i*boxW,     j*boxH );
           glVertex2f( (i+1)*boxW,     j*boxH );
           glVertex2f( (i+1)*boxW, (j+1)*boxH );
           glVertex2f(     i*boxW, (j+1)*boxH );
         glEnd();
      }
   }

   // Draw the single large box on the right half of the
   // screen using a color slected by the user from the
   // left half of the screen.
   glColor3f(bigBoxR, bigBoxG, bigBoxB);
   glBegin(GL_QUADS);
     glVertex2f(windowWidth/2.0, 0.0);  // a square
     glVertex2f(windowWidth, 0.0);
     glVertex2f(windowWidth, windowHeight);
     glVertex2f(windowWidth/2.0, windowHeight);
   glEnd();

   glutSwapBuffers();
}//display()


/* The reshape() callback function */
void reshape(int width, int height)
{
   windowWidth  = width;
   windowHeight = height;

   printf("We got a \"reshape(%d, %d)\" event\n", width, height);
   display();
}//reshape()


/* The mouse() callback function */
// This function reads the color of the pixel
// where the mouse is clicked down on.
void mouse(int button, int state, int x, int y)
{
   float mouseColor[3];

   if ( state == GLUT_DOWN )
   {  // read just one pixel, put the result in the mouseColor[] array
      glReadPixels(x, windowHeight-y, 1, 1, GL_RGB, GL_FLOAT, (void*)mouseColor);

      bigBoxR = mouseColor[0];
      bigBoxG = mouseColor[1];
      bigBoxB = mouseColor[2];
   }

   printf("We got a \"mouse(%d, %d, %d, %d)\" event\n", button, state, x, y);
   glutPostRedisplay();
}//mouse()
